/*
 * AServer Support:
 *
 * Implements a a single aserver connection to a named service.
 * 
 */

#include <stdlib.h>

#include "aserver.h"

#define CONN_TABLE_SIZE      1

static conn_id conn_table[CONN_TABLE_SIZE];
static int connNum;
static as_bool connected = AS_FALSE;
static void (*error_handler) (asrc);


static void abort_reply (int c_num)
{
	packet_hdr abort_hdr;
	int ap_num = as_set_dest (abort_hdr, c_num);
	
	as_hdr_set_len (abort_hdr, 0);
	as_hdr_set_type (abort_hdr, ASPT_DISCONNECT_REPLY | ASPH_EOM_MASK);
	as_hdr_set_p_byte (abort_hdr, 0);
	
	as_send_packet (ap_num, abort_hdr, NULL, NULL);
}


asrc aserver_start (Channel *chan_aserv_in, Channel *chan_aserv_out, char* service, void (*error_func) (asrc))
/*
 * Make a connection to the named service using the channels for an access point
 * Register and error handling function for later send/receive use.
 */
{
	asrc res;
  
	res = as_apstart(chan_aserv_in, chan_aserv_out);
	if (res != ASRC_SUCCESS)
		return res;

	res = as_setconntable(CONN_TABLE_SIZE, conn_table);
	if (res != ASRC_SUCCESS)
		return res;

	res = asc_connect(service, "", 0, &connNum, NULL);
	if (res != ASRC_SUCCESS)
		return res;

	error_handler = error_func;
	connected = AS_TRUE;
	return res;
}


asrc aserver_finish (void)
/*
 * Finish with the aserver connection (if it exists)
 */
{
    if (AS_BOOL_TO_INT(connected))
		asc_disconnect(connNum, NULL);
	return ASRC_SUCCESS;
}


void aserver_send (void *o, unsigned long os)
/*
 * Send a message to our connection
 */
{
	asrc res;

	res = as_sendmessage (connNum, os, o, NULL);
	if (res != ASRC_SUCCESS)
		error_handler (res);
}


void aserver_receive (void *i, unsigned long *is)
/*
 * Receive a message from our connection
 */
{
	asrc res;

	res = as_recmessage (connNum, is, i);

	if (res != ASRC_SUCCESS) {
		if (res == ASRC_GOT_ABORT_REQ) {
			abort_reply (connNum);
		}
		error_handler (res);
	}
}

