#ifndef _ASERVER
#define _ASERVER

#include <aslib.h>

class CAserver;

// A single packet
class APacket {
	friend class CAserver;
	int			ap_num;
	packet_hdr	hdr;
	unsigned char	data[MAX_PACKET_SIZE];
};

// A connection
class AConnection {
	friend class CAserver;                                  
	BOOL		m_bConnected;	// Connection made
    BYTE*		m_pRecData;		// Connection receive buffer
    unsigned	m_nRecSize;		// Connection receive buffer size
    unsigned	m_nRecNum;		// Num bytes currently in receive buffer

    AConnection () {}
    void ArrayInit (int nMaxRecMessageSize)
    	{ m_bConnected = FALSE;
    	  m_pRecData = new BYTE[nMaxRecMessageSize];
    	  m_nRecNum = 0;
		  m_nRecSize = nMaxRecMessageSize;}        
	~AConnection ()
		{ delete [] m_pRecData; }
};


class CAserver {

private:
	static CAserver*	s_active;      	// The active instance that the callback uses
	static BOOL			s_SIGINT;		// Call back has received SIGINT
	static BOOL			s_criticalFIFO;		// Critical section, packet FIFO is volatile
	static BOOL			s_criticalProcess;	// Critical section, processing packet must be completed
	static BOOL			s_criticalSend;		// Critical section, send message must be completed

// FIFO buffer of packets
	static unsigned		s_nR, s_nW;		// Read and Write indexes
	static unsigned		s_nSize;		// Buffer size
	static APacket*		s_pFifo ;		// The FIFO buffer

										// Static functions available, required for use in a callback
	static asrc 		s_AddPacket (int ap_num, packet_hdr hdr, unsigned char* data);
	static asrc			s_GetPacket (int *ap_num, packet_hdr hdr, unsigned char* data);
	static BOOL			s_Empty ();

	static void 		s_Callback (short aserverEvent);
	static void			s_ProtocolFail (const char* message, asrc result = ASRC_SUCCESS, packet_hdr header = NULL);
	static void 		s_ProcessBuffer ();
	
    unsigned			m_nConnect;		// Keep track of number of connections
    unsigned			m_nMaxConnect;	// Maxiumum specified
	conn_id*			m_pConnTable;	// ASERVER connection table
	BOOL				m_bClientLock;	// Client receive buffer lock
	AConnection*		m_pConnects;	// Array of connections

private:
	virtual BOOL RecMessage (int c_num, const void* pMessage, UINT size) = 0;
	virtual void ProtocolFail (const char* errorText) = 0;
	virtual void ConnectionInfo (int c_num, int result, const packet_hdr hdr, BOOL sigInt, const char* service, const char* serviceParam) = 0;

public:	
	CAserver (unsigned nMaxConnect, unsigned nMaxRecMessageSize, unsigned nRecCommonFIFO);
	~CAserver ();
	void RecMessageCompleted ();
	void SendMessage (int c_num, const void* data, unsigned size);
	static void s_AsyncHold (void);
};

#endif /* _ASERVER */
